#!/usr/bin/env groovy
package siena;
import siena.*;

println("extractFPFNplotData");

if (this.args.size() < 4){
  println("usage: extractClientFPFNplotData mergedLogFileName fn-data-series fp-data-series window-millisec");
  return;
}
for (a in this.args) {
  println("Argument: " + a);
}

logFile = new File(args[0]);
fnFile = new File(args[1]);
fnFile.write("");
fpFile = new File(args[2]);
fpFile.write('"time"\t"FP% at client"\t"FP% at server (no BF match)"\t"FP% total"\t"TOTAL FP DRP-packets"\t"TOTAL FP no BF DRP-packets"\t"TOTAL DRP"\n');
int period = Integer.parseInt(args[3]); // size of the moving average window in milliseconds
List lines = logFile.readLines();
Map filters = new HashMap();

int iteration = 1;
int falseNeg = 0;
int falsePos = 0;
int bfFalsePos = 0;
int msgs = 0;
int pubs = 0;
int received = 0;
int fpUnneededPacket = 0;
int bfFpUnneededPacket = 0;
int drp = 0;
Map fNegs = new HashMap();

//println(cmd);
pattern = /(\d+) (\w+) (\w+) .*\d: (\{.+\}){0,1}/;
for(i in 0..lines.size()-1){
    matcher = (lines[i] =~ pattern);
    //println(lines[i]);    
    if (matcher) {
        //println(matcher[0][0]);        
        // check if moving average period is over, print results and set counters to 0
        int time = Integer.parseInt(matcher[0][1]);        
        while( time > (iteration * period)){
            if (falseNeg != 0){
                fnFile.append(iteration * period + "\t" + falseNeg / (falseNeg + received) + "\t" + falseNeg + "\t" + received + "\n");
            } else {
                fnFile.append(iteration * period + "\t" + 0 + "\t" + falseNeg + "\t" + received + "\n");
            }
            if (drp != 0){
                fpFile.append(iteration * period  + "\t" + (fpUnneededPacket - bfFpUnneededPacket)/ drp + "\t" + bfFpUnneededPacket / drp + "\t" + 
                + fpUnneededPacket / drp + "\t" + fpUnneededPacket + "\t" + bfFpUnneededPacket + "\t" + drp + "\n");
            } else {
                fpFile.append(iteration * period  + "\t" + 0 + "\t" + fpUnneededPacket + "\t" + bfFpUnneededPacket + "\t" + drp + "\n");
            }
            falseNeg = 0;
            falsePos = 0;
            bfFalsePos = 0;
            msgs = 0;
            pubs = 0;
            fpUnneededPacket = 0;
            bfFpUnneededPacket = 0;
            drp = 0;
            received = 0;
            fNegs.clear();
            iteration++;
        }
        switch (matcher[0][3]) {
            case "subscribing" :
                SENPBuffer pckt = new SENPBuffer(matcher[0][4].getBytes());
                Filter f = pckt.decodeFilter();
                List nodeFilters = filters.get(matcher[0][2]);
                if (nodeFilters == null){
                    nodeFilters = new ArrayList();
                    filters.put(matcher[0][2], nodeFilters);
                }
                //println("Filters for ${matcher[0][2]}: ${filters.get(matcher[0][2])}");
                // here we implement the same logic as in Siena
                // that is, if a subscription is covered by a more general one
                // only the most general is kept
                boolean toBeAdded = true;
                List toBeRemoved = [];
                for(filter in nodeFilters){
                    if (Covering.covers(f,filter)){
                        toBeRemoved.add(filter);                        
                    } else if (Covering.covers(filter,f)) {
                        toBeAdded = false;
                    }
                }
                // remove actual unneded filters
                for(filter in toBeRemoved){
                    nodeFilters.remove(filter);
                }
                if(toBeAdded){
                    nodeFilters.add(f);
                }
                //println("Filters for ${matcher[0][2]}: ${filters.get(matcher[0][2])}");
                break;   
            case "publishing" :
                pubs++;
                //println("msg line: " + matcher[0][0]);                
                SENPBuffer pckt = new SENPBuffer(matcher[0][4].getBytes());
                Notification n = pckt.decodeNotification();
                msgId = n.getAttribute("msgId");
                //println("msgId: ${msgId}");
                // look for matching subscriptions
                for(node in filters.keySet()){
                    //println(node);
                    int k = 0;
                    boolean found = false;
                    while(k < filters.get(node).size() && !found){
                        filter = filters.get(node)[k];
                        //println(filter);
                        if (Covering.apply(filter, n)){
                            msgs++;
                            //println("${matcher[0][1]} ${node} filter ${filter} covers ${n}");
                            // here we search the rest of the log to find if msg was delivered
                            String getRegExp = "\\d+ " +  node + ' getting.*' + msgId ;
                            int j = (i > 100) ? i - 100 : 0 ; 
                            // changed: we look 100 lines in the past and forward in the rest of the log as we cannot 
                            // really trust log timings and delivery timings                                                      
                            while(j < lines.size() && j < i + 100 && !found){                                                                
                                match = (lines[j] =~ getRegExp);
                                if (match){
                                   found = true;
                                   //println("Match: ${lines[j]}");                                    
                               }
                               j++;
                           }
                           if (!found) {
                             falseNeg++;
                             println("False Negative: ${time} ${n} was not delivered to ${node} for filter ${filter}");
                             // we keep track of all falseNegs
                             // the idea is that we want ot avoid counting an undelivered packet as both 
                             // a false Pos and a false Neg (e.g., on a predicate change). 
                             // So we keep false pos but not false negs if they overlap
                             List msgFPs = [];
                             if (fNegs.get(msgId) == null){
                                 fNegs.put(msgId, msgFPs);
                             }
                             msgFPs.add(node);
                             println("fNegs: ${fNegs}");
                           }                           
                        }
                     k++;
                    }                    
                }   
                // look for false positives
                int j = i ; 
                // changed: we only look ahead assuming we've restored causality order at least for DRP packets                
                String fpRegExp = "\\d+ (\\w+) " + '.*false positive.*' + msgId ;
                // a list to store all unintended recipients (based on actual predicates) for DRP packets
                List fpNodeIds = [];
                // a list to store all unintended recipients (based on Bloom Filters) for DRP packets
                List bfFpNodeIds = [];
                // we have fpNodeIds.contains(bfFpNodeIds) i.e. |fpNodeIds| >= |bfFpNodeIds|
                while(j < lines.size() && j < i + 100){                      
                        match = (lines[j] =~ fpRegExp);
                         if (match){                             
                             println("False Positive: ${lines[j]}");
                             if(lines[j].contains("no matching BF")){
                                 // then this is false positive due to latency 
                                 // (or of course any screw up I made in the implementation)
                                 // as no matching BF was found at destination                                 
                                 bfFalsePos++;
                                 // we mark current destination as FP destination
                                 bfFpNodeIds.add(match[0][1]);
                             }
                             // anyhow it adds to the list of false positives
                             falsePos++;                                 
                             fpNodeIds.add(match[0][1]);
                             // adjust FN count so as to avoid counting undelivered msgs as both FP and FN
                             if ((msgFPs = fNegs.get(msgId)) != null){
                                 //TODO: here we have ServerID and not clientID, find a solution to map them
                                 if(msgFPs.contains(match[0][1])){
                                     falseNeg--;
                                     println("Removing FN on msg ${msgId} to ${node} -> counted as FP");
                                 }
                             }
                         }
                  j++;
                }                
                if (fpNodeIds.size() > 0) {
                    println("fpNodeIds: ${fpNodeIds} ");
                    println("bfFpNodeIds: ${bfFpNodeIds} ");
                    // here we count how many uneeded hops were traversed
                    // we measure percentage of fp as no. unneded hops  / total hops traversed by all DRP msg (including to other destinations and for other events)
                     int k = i ; 
                     // changed: we only look ahead assuming we've restored causality order at least for DRP packets                    
                    String drpRegExp = '(\\d+).+processRequest.+DRP.+recList\\=\\{([\\w ]+)\\}.*' + msgId ;
                    while(k < lines.size() && k < i + 100){                      
                            match = (lines[k] =~ drpRegExp);
                             if (match){                                 
                                 //println(match[0]);
                                 // recList={S3 S6 }
                                 nodeIdsMatcher = (match[0][2] =~ /(\w+)/);
                                 if (nodeIdsMatcher){
                                     //println("nodeIdsMatcher: ${nodeIdsMatcher}");
                                     List packetNodes = [];
                                     for(p in 0..nodeIdsMatcher.getCount()-1){
                                         packetNodes.add(nodeIdsMatcher[p][1]);
                                     }
                                     println("packetNodes: ${packetNodes}");
                                     if (packetNodes.findAll({!fpNodeIds.contains(it)}).size() == 0){
                                     // if nodes are only the ones among fpNodeIds we got ourself an unneeded packet
                                     fpUnneededPacket++;
                                     println("fpNodeIds.contains(packetNodes)");
                                     }
                                     if (packetNodes.findAll({!bfFpNodeIds.contains(it)}).size() == 0){
                                     // if nodes are only the ones among bfFpNodeIds we got ourself an unneeded packet due to latency                                     
                                     bfFpUnneededPacket++;
                                     println("bfFpNodeIds.contains(packetNodes)");
                                     }
                                 }
                                 // we add to drp count if we're getting outside of the current window
                                 // and "consume" (e.g. delete) the line so that it is not added to other windows
                                 // this induces some error as DRP packets are all considered in the same window as 
                                 // their originating PUB
                                 // (at least we prevent FP percentages of more than 100%)
                                 int lineTime = Integer.parseInt(match[0][1]);
                                 if(lineTime > iteration * period) {
                                     drp++;
                                     lines.remove(k);
                                     // prevent line skipping
                                     k--;
                                 }
                                 // otherwise we don't add to drp as it's taken into account by rightful counter
                             }                             
                      k++;
                    }
                }                
                break;
            case "unsubscribing" :                
                List nodeFilters = filters.get(matcher[0][2]);                
                SENPBuffer pckt = new SENPBuffer(matcher[0][4].getBytes());
                Filter f = pckt.decodeFilter();                
                nodeFilters.remove(f);                                
                break;
            case "getting" :
                received++;
        }
    }
    // counter for DRP packets
    if (lines[i] =~ /processRequest: senp\{version\=1 method\=\"DRP\"/){
        drp++;
    }
}
if (falseNeg != 0){
    fnFile.append(iteration * period + "\t" + falseNeg / (falseNeg + received) + "\t" + falseNeg + "\t" + received + "\n");
} else {
    fnFile.append(iteration * period + "\t" + 0 + "\t" + falseNeg + "\t" + received + "\n");
}
if (drp != 0){
    fpFile.append(iteration * period  + "\t" + (fpUnneededPacket - bfFpUnneededPacket)/ drp + "\t" + bfFpUnneededPacket / drp + "\t" + 
    + fpUnneededPacket / drp + "\t" + fpUnneededPacket + "\t" + bfFpUnneededPacket + "\t" + drp + "\n");
} else {
    fpFile.append(iteration * period  + "\t" + 0 + "\t" + fpUnneededPacket + "\t" + bfFpUnneededPacket + "\t" + drp + "\n");
}
//println(filters);
