//
// -*- C++ -*-
//
//  This file is part of WEEVILGEN.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: sienaclient.cc,v 1.1.1.1 2004/11/02 23:36:43 ywang Exp $
//
// implementation of a random, communicating siena client
#include <cassert>
#include <cstring>
#include <string>
#include <weevil/weevil.h>

using namespace ssim;
using namespace std;
using namespace weevil;

class connection{
public:
  std::string src;
  std::string dest;  
};


/**
 * sequential process class
 */
class ServerLinkConfig : public weevil::WeevilTProcess
{
  /**
   * properties from m4 configuration
   */
  
  //the list of links to be enabled across servers
  vector<connection*> connections;

public:
  /**
   * sequential process
   */  
   ServerLinkConfig(char* id);
   
  virtual void main();

   void neighbors( std::string list ){
    if ( list.compare("") != 0 ){
      int begin = 0;
      int end = list.find_first_of(",", begin);
      while((end != string::npos) && (begin != string::npos)){
          connection* new_connection = new connection();
          new_connection->src = list.substr(begin, end-begin);
          begin = list.find_first_not_of(",", end + 1);
          end = list.find_first_of("|", begin);   
          if (end != string::npos){
              new_connection->dest = list.substr(begin, end-begin);
              connections.push_back(new_connection);
              begin = list.find_first_not_of("|", end + 1);
              end = list.find_first_of(",", begin);
          } else {
              new_connection->dest = list.substr(begin, list.length());
          }         
      }      
      }
    }  
};

/**
 * method implementation
 */
ServerLinkConfig :: ServerLinkConfig(char* id){
  //Clear the vectors first: after the execution of this construction,
  //the process properties(including m_sub_neighbors and
  //m_pub_neighbors) will be assigned.
  connections.clear();  
}

void ServerLinkConfig :: main()
{  
  //instantiate all connections
  for (int i = 0; i < connections.size(); i ++){
      ostringstream str_stream;
	  str_stream << "`LINK" << "(" << connections[i]->src << "," << connections[i]->dest << ")'";
      string str_msg = str_stream.str();
      const char* msg = str_msg.c_str();
      workload_output( msg, "", "");
  }
}
