package siena;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import siena.comm.GenericSenderFactory;
import siena.comm.PacketSender;
import siena.comm.PacketSenderFactory;
import weevil.Actor;

public class LinkActor implements Actor{
	

	private Map nodes = new HashMap();
	private PacketSenderFactory psf = new GenericSenderFactory();
	/**
	 * GT: used finer time granularity
	 */
    static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS Z");

	public LinkActor( String arg ){
		//parse arg
		String[] configparser = arg.split(" ");				
		////////////////system specific/////////////
		int i = 0;
		if (configparser.length > 3){			
			while (i < configparser.length){
				String name = configparser[i];
				nodes.put(name, new Server(name, configparser[i+1], configparser[i+2], Integer.parseInt(configparser[i+3])));
				i += 4;
			}
		}	
		Iterator ite = nodes.keySet().iterator();
		while (ite.hasNext()){
			String name = (String) ite.next();
			Server node = (Server) nodes.get(name);
			System.out.println("Server " + name + ": " + node.id + " " + node.address );
		}	 	
	}

	public void action(String arg){
		String command = arg.substring(0, arg.indexOf('('));
		String argument = arg.substring(arg.indexOf('(')+1, arg.lastIndexOf(')'));
		if (command.equals("LINK")){
			linkAction(argument);
		}
		else if (command.equals("CUT")){
			cutAction(argument);
		}
		
	}

	public void stop(){

	}

    
	public void linkAction(String arg){
		String src = arg.substring(0, arg.indexOf(","));
		String dest = arg.substring(arg.indexOf(",") +1);
		System.out.println("At time(i) " + sdf.format(new Date()) + ", " + "linking " + src + " to " + dest);
		link((Server) nodes.get(src), (Server) nodes.get(dest));		
	}

	
	public void cutAction(String arg){
		String src = arg.substring(0, arg.indexOf(","));
		String dest = arg.substring(arg.indexOf(",") +1);
		System.out.println("At time(i) " + sdf.format(new Date()) + ", " + "cutting link from " + src + " to " + dest);
		cut((Server) nodes.get(src), (Server) nodes.get(dest));
	}
	
	
	public void link(Server source, Server dest){
		SENPPacket pack = SENPPacket.allocate();
		pack.id = source.id.getBytes();
		pack.handler = source.address.getBytes();
		pack.method = SENP.CNF;
		pack.ttl = SENP.DefaultTtl;	
		try{
		PacketSender sd = psf.createPacketSender(dest.address);
		sd.send(pack.buf,pack.encode());
		} catch (Exception e)  {
			System.err.println("At time(i) " + sdf.format(new Date()) + ", " + "caught exception connecting " + source.id + " to " + dest.id + " " + dest.address);
			e.printStackTrace();
		}
	}
	
	public void cut(Server source, Server dest){
		SENPPacket pack = SENPPacket.allocate();
		SENPPacket pack2 = SENPPacket.allocate();
		pack.id = source.id.getBytes();
		pack2.id = dest.id.getBytes();
		pack.method = SENP.CNF;
		pack2.method = SENP.CNF;
		pack.ttl = SENP.DefaultTtl;	
		pack2.ttl = SENP.DefaultTtl;
		try{
		PacketSender sd = psf.createPacketSender(dest.address);
		PacketSender sd2 = psf.createPacketSender(source.address);
		sd.send(pack.buf,pack.encode());
		sd2.send(pack2.buf,pack2.encode());
		} catch (Exception e)  {
			System.err.println("At time(i) " + sdf.format(new Date()) + ", " + "caught exception connecting " + source.id + " to " + dest.id + " " + dest.address);
			e.printStackTrace();
		}
		
	}
	
	class Server {
		public String protocol, ip, id, address;
		public int port;
		public Server(String id, String protocol, String ip, int port) {
			super();			
			this.ip = ip;
			this.id = id;
			this.port = port;
			this.address = protocol + ":" + ip + ":" + port;			
		}
		
		
		
	}
}
