package weevil;
/** a generic client workload actor
 *
 *	This interface implements a generic client actor who will translate
 *	a workload line into system commands and send it to the system.
 *
 *	This interface should be implemented in your system specific client
 *	workload actor.
 *    
 */
public interface Actor{
	/**
	 *	This method defines the actions for each kind of workload
	 *
	 *	In your derived class, you must provide method definition for this
	 *	method. In the method implementation, you need to parse this workload
	 *	line, and specify the system specific commands according to the type of
	 *	this workload line.
	 *	@param arg represents a line of workload
	 */
	public void action(String arg);
	/**
	 *	This method defines the actor's action when it finishes the whole workload
	 *
	 *	In your derived class, you must provide method definition for this
	 *	method. In the method implementation, you could specify some actions
	 *	to conduct when the client finishes the workload.
	 */
	public void stop();
}
